# (Steffen Kothe / CM SAF 2018-07-26)

shinyServer(function(input, output, session) {
  
  observeEvent(input$exit, {
    stopApp()
  })
  
  output$summary1 <- renderPrint({
    ncinfo(infile)
  })
  
  output$summary2 <- renderPrint({
    ncinfo(infile,"m")
  })
  
  output$about <- renderPrint({
    cat("The CMSAF Visualizer is part of the CM SAF R Toolbox.","\n")
    cat("The script helps you to visualize 1D-timeseries and 2D-maps.","\n")
    cat("Your NetCDF data have to follow CF conventions and the data","\n")
    cat("have to be on a rectangular grid with longitude, latitude","\n")
    cat("and time dimension.","\n")
    cat("This version (July 2018) was tested with the cmsaf R-package","\n")
    cat("in version 1.9.2.","\n")
    cat("\n")
    cat("Suggestions for improvements and praise for the developers","\n")
    cat("can be send to contact.cmsaf@dwd.de.","\n")
    cat("\n")
    cat("                              - Steffen Kothe - 2018-07-25 -","\n")
    cat("\n")
    cat("\n")
  })
  
  url <- a("http://www.cmsaf.eu/tools", href="http://www.cmsaf.eu/tools")
  output$link <- renderUI({
    tagList("URL link:", url)
  })
  
  output$tipps <- renderPrint({
    cat("When you save a figure, please add the file extension .png !","\n")
    cat("\n")
    cat("If saving an image fails, try right-click plus 'save image as'.","\n")
    cat("\n")
  })
  
  output$myImage <- renderImage({
    # A temp file to save the output.
    # This file will be removed later by renderImage
    outfile <- tempfile(fileext='.png')
    
    # prepare ticks and date formats
    dum_tick <- seq(1,length(date.time),length.out=input$ticknumber)
    dum_tick2 <- NULL
    for (j in 2:length(dum_tick)){
      dummy <- seq(dum_tick[j-1],dum_tick[j],length.out=4)
      if (j>2&j!=length(dum_tick))(dummy <- dummy[2:4])
      dum_tick2 <- append(dum_tick2,dummy)
    }
    if (input$dateformat==1)(date.lab <- format(date.time[dum_tick],"%Y")) 
    if (input$dateformat==2)(date.lab <- format(date.time[dum_tick],"%Y-%m"))
    if (input$dateformat==3)(date.lab <- format(date.time[dum_tick],"%Y-%m-%d"))
    
    if(as.logical(input$analyze)){
      # Set the size of the output window
      iwidth  <- 650
      iheight <- 800
      
      png(outfile, width=iwidth, height=iheight)
      # Analyze Timeseries
      
      # Set the number of rows and columns
      nrow <- 3
      ncol <- 2
      
      field <- data[,,input$slider1[1]:input$slider1[2]]
      date.time2 <- date.time[input$slider1[1]:input$slider1[2]]
      
      # Create vectors of the months and years, respectively
      timemonth <- format(date.time2,"%m")
      timeyear <- format(date.time2,"%Y")
      nt <- length(date.time2)
      
      # Create vectors of the months and years, respectively
      timemonth.in <- format(date.time2,"%m")
      timeyear.in <- format(date.time2,"%Y")
      startyear <- timeyear.in[1]
      startmonth <- timemonth.in[1]
      
      title <- varname
      varlabel <- paste(title," [",unit,"]",sep="")
      
      # The function tapply is very useful for operations that 
      # operate along the time axis, e.g., calculating mean monthly values
      field.monmean <- tapply(field,timemonth,mean,na.rm=TRUE)
      field.monmax <- tapply(field,timemonth,max,na.rm=TRUE)
      field.monmin <- tapply(field,timemonth,min,na.rm=TRUE)
      field.monsd <- tapply(field,timemonth,sd,na.rm=TRUE)
      
      # Annual mean is only calculated if data for all 12 month are available
      field.annmean <- tapply(field,timeyear,mean,na.rm=TRUE)
      # Include only those years with 12 months on data
      years <- dimnames(field.annmean)[[1]]
      nyears <- length(years)
      nmonth <- vector(mode="numeric",length=nyears)
      for (i in 1:nyears) {
        nmonth[i] <- length(which(timeyear==years[i]))
      }
      ind <- which(nmonth < 12)
      field.annmean[ind] <- NA
      
      # Calculate anomalies
      field.ano <- vector(mode="numeric",length=nt)
      field.relano <- vector(mode="numeric",length=nt)
      for (j in 1:nt){
        field.ano[j] <- field[j] - field.monmean[timemonth[j]]
        field.relano[j] <- field.ano[j]/field.monsd[timemonth[j]]
      }
      
      # set the number of rows and columns of the plot
      par(mfrow = c(nrow,ncol))
      
      # Determine the min and max of the plotrange
      pmin <- min(field,na.rm=TRUE)
      pmax <- max(field,na.rm=TRUE)
      drange <- pmax - pmin
      
      #plot the data
      plot(date.time2,field, ylab=varlabel, xlab="",
           main=paste(title,", ",format(lat,digits=4,nsmall=2)," N, ",
                      format(lon,digits=4,nsmall=2)," E",sep=""),
           ylim=c(pmin,pmax),type="l")
      
      # calculate the linear trend of the original data
      x <- c(1:nt)
      model <- lm(field~x,na.action=na.exclude)
      lines(date.time2,predict(model),col=input$integer,lwd=2.0)
      conf <- confint(model,"x",level=0.95)
      trend <- model$coeff["x"]*12.
      lconf <- conf[1]*12.
      uconf <- conf[2]*12
      
      mean.out <- format(mean(field,na.rm=TRUE),digits=3,nsmall=1)
      trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                         format(uconf,digits=2,nsmall=2),"] ",sep="")
      
      # Determine the x-location of the text in date format
      xtext <- as.Date(paste(startyear,startmonth,01,sep="-"))
      
      text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
      text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
      
      #--------------------------------------------------
      #Plot the monthly mean seasonal cycle
      months <- seq(1:12)
      # Determine the min and max of the plotrange
      pmin <- min(field.monmin,na.rm=TRUE)
      pmax <- max(field.monmax,na.rm=TRUE)
      
      plot(months,field.monmax,type="n",main="Average Seasonal Cycle",
           ylab=varlabel,xlab="Months",ylim=c(pmin,pmax))
      lines(months,field.monmax)
      lines(months,field.monmin)
      polygon(c(months,rev(months)),c(field.monmin,rev(field.monmax)),col=input$integer)
      lines(months,field.monmean,lwd=2)
      
      #--------------------------------------------------
      # Plot the monthly anomalies
      pmin <- min(field.ano,na.rm=TRUE)
      pmax <- max(field.ano,na.rm=TRUE) + 0.15 * (max(field.ano,na.rm=TRUE) - min(field.ano,na.rm=TRUE))
      drange <- pmax - pmin
      plot(date.time2,field.ano, ylab=varlabel, xlab="",
           main="Monthly anomalies",
           ylim=c(pmin,pmax), type="l")
      abline(h=0,lwd=1.0,col="gray40")
      
      # calculate the linear trend of the anomaly data
      x <- c(1:nt)
      model <- lm(field.ano~x,na.action=na.exclude)
      lines(date.time2,predict(model),col=input$integer,lwd=2.0)
      conf <- confint(model,"x",level=0.95)
      trend <- model$coeff["x"]*12.
      lconf <- conf[1]*12.
      uconf <- conf[2]*12
      
      trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                         format(uconf,digits=2,nsmall=2),"] ",sep="")
      
      # Determine the x-location of the text in date format
      xtext <- as.Date(paste(startyear,startmonth,01,sep="-"))
      
      text(xtext,pmax-0.01*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
      
      
      #--------------------------------------------------
      # Boxplot of the time series
      # Define the months as a categorical variable
      month_cat <- factor(timemonth)
      plot(month_cat,field,main="Box Plot",ylab=varlabel,xlab="Months")
      
      #--------------------------------------------------
      # Plot the annual means
      plot(as.integer(years)+0.5,field.annmean,type="p",main="Annual Means",
           ylab=varlabel,xlab="",pch=19)
      abline(h=mean(field.annmean,na.rm=T),lwd=1.0,col="gray40")
      
      #--------------------------------------------------
      # Plot a histogram of the data
      hist_field <- hist(field,breaks=20,xlab=varlabel,
                         main=paste("Histogram of ",title,sep=""),col=input$integer)
      box(col="gray20",lwd=1)
      
      on.exit(dev.off())
    } else {
      
      # Generate the PNG with different line types
      if (input$checkGroup==1){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        plot(date.time,data,type="l",xlim=date.time[input$slider1],ylim=input$slider2,
             col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
        abline(h=0,lwd=1,col="gray")
        grid(NA,NULL,lwd=0.8)
        abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
        points(date.time,data,type="l",xlim=date.time[input$slider1],ylim=input$slider2,
               col=input$integer,lwd=linesize)
        axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
        rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
        axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
        box(col="gray20",lwd=1)
        if (as.logical(input$trend)){
          # calculate the linear trend
          x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
          model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
          lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
          conf <- confint(model,"x",level=0.95)
          trend <- model$coeff["x"]*12.
          lconf <- conf[1]*12.
          uconf <- conf[2]*12
          
          mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
          trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                             format(uconf,digits=2,nsmall=2),"] ",sep="")
          xtext <- as.Date(date.time[input$slider1[1]])
          
          pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          drange <- pmax-pmin
          text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
          text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
        }
        mtext(input$text2)
        on.exit(dev.off())
      }
      
      if (input$checkGroup==2){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        plot(date.time,data,type="p",xlim=date.time[input$slider1],ylim=input$slider2,
             col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
        abline(h=0,lwd=1,col="gray")
        grid(NA,NULL,lwd=0.8)
        abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
        points(date.time,data,type="p",xlim=date.time[input$slider1],ylim=input$slider2,
               col=input$integer,lwd=linesize)
        axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
        rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
        axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
        box(col="gray20",lwd=1)
        if (as.logical(input$trend)){
          # calculate the linear trend
          x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
          model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
          lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
          conf <- confint(model,"x",level=0.95)
          trend <- model$coeff["x"]*12.
          lconf <- conf[1]*12.
          uconf <- conf[2]*12
          
          mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
          trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                             format(uconf,digits=2,nsmall=2),"] ",sep="")
          xtext <- as.Date(date.time[input$slider1[1]])
          
          pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          drange <- pmax-pmin
          text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
          text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
        }
        mtext(input$text2)
        on.exit(dev.off())
      }
      
      if (input$checkGroup==3){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        plot(date.time,data,type="o",xlim=date.time[input$slider1],ylim=input$slider2,
             col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
        abline(h=0,lwd=1,col="gray")
        grid(NA,NULL,lwd=0.8)
        abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
        points(date.time,data,type="o",xlim=date.time[input$slider1],ylim=input$slider2,
               col=input$integer,lwd=linesize)
        axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
        rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
        axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
        box(col="gray20",lwd=1)
        if (as.logical(input$trend)){
          # calculate the linear trend
          x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
          model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
          lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
          conf <- confint(model,"x",level=0.95)
          trend <- model$coeff["x"]*12.
          lconf <- conf[1]*12.
          uconf <- conf[2]*12
          
          mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
          trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                             format(uconf,digits=2,nsmall=2),"] ",sep="")
          xtext <- as.Date(date.time[input$slider1[1]])
          
          pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          drange <- pmax-pmin
          text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
          text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
        }
        mtext(input$text2)
        on.exit(dev.off())
      }
      
      if (input$checkGroup==4){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        plot(date.time,data,type="s",xlim=date.time[input$slider1],ylim=input$slider2,
             col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
        abline(h=0,lwd=1,col="gray")
        grid(NA,NULL,lwd=0.8)
        abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
        points(date.time,data,type="s",xlim=date.time[input$slider1],ylim=input$slider2,
               col=input$integer,lwd=linesize)
        axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
        rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
        axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
        box(col="gray20",lwd=1)
        if (as.logical(input$trend))(points(date.time,fitted,type="l",col="gray20",lwd=linesize))
        mtext(input$text2)
        on.exit(dev.off())
      }
      
      if (input$checkGroup==5){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        plot(date.time,data,type="h",xlim=date.time[input$slider1],ylim=input$slider2,
             col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
        abline(h=0,lwd=1,col="gray")
        grid(NA,NULL,lwd=0.8)
        abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
        points(date.time,data,type="h",xlim=date.time[input$slider1],ylim=input$slider2,
               col=input$integer,lwd=linesize)
        axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
        rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
        axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
        axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
        box(col="gray20",lwd=1)
        if (as.logical(input$trend)){
          # calculate the linear trend
          x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
          model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
          lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
          conf <- confint(model,"x",level=0.95)
          trend <- model$coeff["x"]*12.
          lconf <- conf[1]*12.
          uconf <- conf[2]*12
          
          mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
          trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                             format(uconf,digits=2,nsmall=2),"] ",sep="")
          xtext <- as.Date(date.time[input$slider1[1]])
          
          pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
          drange <- pmax-pmin
          text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
          text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
        }
        mtext(input$text2)
        on.exit(dev.off())
      }
    } # end if analyze
    
    # Return a list containing the filename
    list(src = outfile,
         contentType = 'image/png',
         width = iwidth,
         height = iheight,
         alt = "This is alternate text")
  }, deleteFile = TRUE)
  
  # save image
  output$downloadPlot <- downloadHandler(
    filename = function(){
      paste("plot","png",sep=".")},
    contentType = "image/png",
    content = function(file) {
      
      # prepare ticks and date formats
      dum_tick <- seq(1,length(date.time),length.out=input$ticknumber)
      dum_tick2 <- NULL
      for (j in 2:length(dum_tick)){
        dummy <- seq(dum_tick[j-1],dum_tick[j],length.out=4)
        if (j>2&j!=length(dum_tick))(dummy <- dummy[2:4])
        dum_tick2 <- append(dum_tick2,dummy)
      }
      if (input$dateformat==1)(date.lab <- format(date.time[dum_tick],"%Y")) 
      if (input$dateformat==2)(date.lab <- format(date.time[dum_tick],"%Y-%m"))
      if (input$dateformat==3)(date.lab <- format(date.time[dum_tick],"%Y-%m-%d"))
      
      if(as.logical(input$analyze)){
        # Set the size of the output window
        iwidth  <- 650
        iheight <- 800
        
        png(file, width=iwidth, height=iheight)
        # Analyze Timeseries
        
        # Set the number of rows and columns
        nrow <- 3
        ncol <- 2
        
        field <- data[,,input$slider1[1]:input$slider1[2]]
        date.time2 <- date.time[input$slider1[1]:input$slider1[2]]
        
        # Create vectors of the months and years, respectively
        timemonth <- format(date.time2,"%m")
        timeyear <- format(date.time2,"%Y")
        nt <- length(date.time2)
        
        # Create vectors of the months and years, respectively
        timemonth.in <- format(date.time2,"%m")
        timeyear.in <- format(date.time2,"%Y")
        startyear <- timeyear.in[1]
        startmonth <- timemonth.in[1]
        
        title <- varname
        varlabel <- paste(title," [",unit,"]",sep="")
        
        # The function tapply is very useful for operations that 
        # operate along the time axis, e.g., calculating mean monthly values
        field.monmean <- tapply(field,timemonth,mean,na.rm=TRUE)
        field.monmax <- tapply(field,timemonth,max,na.rm=TRUE)
        field.monmin <- tapply(field,timemonth,min,na.rm=TRUE)
        field.monsd <- tapply(field,timemonth,sd,na.rm=TRUE)
        
        # Annual mean is only calculated if data for all 12 month are available
        field.annmean <- tapply(field,timeyear,mean,na.rm=TRUE)
        # Include only those years with 12 months on data
        years <- dimnames(field.annmean)[[1]]
        nyears <- length(years)
        nmonth <- vector(mode="numeric",length=nyears)
        for (i in 1:nyears) {
          nmonth[i] <- length(which(timeyear==years[i]))
        }
        ind <- which(nmonth < 12)
        field.annmean[ind] <- NA
        
        # Calculate anomalies
        field.ano <- vector(mode="numeric",length=nt)
        field.relano <- vector(mode="numeric",length=nt)
        for (j in 1:nt){
          field.ano[j] <- field[j] - field.monmean[timemonth[j]]
          field.relano[j] <- field.ano[j]/field.monsd[timemonth[j]]
        }
        
        # set the number of rows and columns of the plot
        par(mfrow = c(nrow,ncol))
        
        # Determine the min and max of the plotrange
        pmin <- min(field,na.rm=TRUE)
        pmax <- max(field,na.rm=TRUE)
        drange <- pmax - pmin
        
        #plot the data
        plot(date.time2,field, ylab=varlabel, xlab="",
             main=paste(title,", ",format(lat,digits=4,nsmall=2)," N, ",
                        format(lon,digits=4,nsmall=2)," E",sep=""),
             ylim=c(pmin,pmax),type="l")
        
        # calculate the linear trend of the original data
        x <- c(1:nt)
        model <- lm(field~x,na.action=na.exclude)
        lines(date.time2,predict(model),col=input$integer,lwd=2.0)
        conf <- confint(model,"x",level=0.95)
        trend <- model$coeff["x"]*12.
        lconf <- conf[1]*12.
        uconf <- conf[2]*12
        
        mean.out <- format(mean(field,na.rm=TRUE),digits=3,nsmall=1)
        trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                           format(uconf,digits=2,nsmall=2),"] ",sep="")
        
        # Determine the x-location of the text in date format
        xtext <- as.Date(paste(startyear,startmonth,01,sep="-"))
        
        text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
        text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
        
        #--------------------------------------------------
        #Plot the monthly mean seasonal cycle
        months <- seq(1:12)
        # Determine the min and max of the plotrange
        pmin <- min(field.monmin,na.rm=TRUE)
        pmax <- max(field.monmax,na.rm=TRUE)
        
        plot(months,field.monmax,type="n",main="Average Seasonal Cycle",
             ylab=varlabel,xlab="Months",ylim=c(pmin,pmax))
        lines(months,field.monmax)
        lines(months,field.monmin)
        polygon(c(months,rev(months)),c(field.monmin,rev(field.monmax)),col=input$integer)
        lines(months,field.monmean,lwd=2)
        
        #--------------------------------------------------
        # Plot the monthly anomalies
        pmin <- min(field.ano,na.rm=TRUE)
        pmax <- max(field.ano,na.rm=TRUE) + 0.15 * (max(field.ano,na.rm=TRUE) - min(field.ano,na.rm=TRUE))
        drange <- pmax - pmin
        plot(date.time2,field.ano, ylab=varlabel, xlab="",
             main="Monthly anomalies",
             ylim=c(pmin,pmax), type="l")
        abline(h=0,lwd=1.0,col="gray40")
        
        # calculate the linear trend of the anomaly data
        x <- c(1:nt)
        model <- lm(field.ano~x,na.action=na.exclude)
        lines(date.time2,predict(model),col=input$integer,lwd=2.0)
        conf <- confint(model,"x",level=0.95)
        trend <- model$coeff["x"]*12.
        lconf <- conf[1]*12.
        uconf <- conf[2]*12
        
        trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                           format(uconf,digits=2,nsmall=2),"] ",sep="")
        
        # Determine the x-location of the text in date format
        xtext <- as.Date(paste(startyear,startmonth,01,sep="-"))
        
        text(xtext,pmax-0.01*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
        
        
        #--------------------------------------------------
        # Boxplot of the time series
        # Define the months as a categorical variable
        month_cat <- factor(timemonth)
        plot(month_cat,field,main="Box Plot",ylab=varlabel,xlab="Months")
        
        #--------------------------------------------------
        # Plot the annual means
        plot(as.integer(years)+0.5,field.annmean,type="p",main="Annual Means",
             ylab=varlabel,xlab="",pch=19)
        abline(h=mean(field.annmean,na.rm=T),lwd=1.0,col="gray40")
        
        #--------------------------------------------------
        # Plot a histogram of the data
        hist_field <- hist(field,breaks=20,xlab=varlabel,
                           main=paste("Histogram of ",title,sep=""),col=input$integer)
        box(col="gray20",lwd=1)
        
        on.exit(dev.off())
      } else {
        
        if (input$checkGroup==1){
          iwidth  <- imagewidth
          iheight <- imageheight
          png(file, width=iwidth, height=iheight)
          par(cex = textsize)
          plot(date.time,data,type="l",xlim=date.time[input$slider1],ylim=input$slider2,
               col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
          abline(h=0,lwd=1,col="gray")
          grid(NA,NULL,lwd=0.8)
          abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
          points(date.time,data,type="l",xlim=date.time[input$slider1],ylim=input$slider2,
                 col=input$integer,lwd=linesize)
          axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
          rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
          axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
          box(col="gray20",lwd=1)
          if (as.logical(input$trend)){
            # calculate the linear trend
            x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
            model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
            lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
            conf <- confint(model,"x",level=0.95)
            trend <- model$coeff["x"]*12.
            lconf <- conf[1]*12.
            uconf <- conf[2]*12
            
            mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
            trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                               format(uconf,digits=2,nsmall=2),"] ",sep="")
            xtext <- as.Date(date.time[input$slider1[1]])
            
            pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            drange <- pmax-pmin
            text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
            text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
          }
          mtext(input$text2)
          on.exit(dev.off())
        }
        
        if (input$checkGroup==2){
          iwidth  <- imagewidth
          iheight <- imageheight
          png(file, width=iwidth, height=iheight)
          par(cex = textsize)
          plot(date.time,data,type="p",xlim=date.time[input$slider1],ylim=input$slider2,
               col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
          abline(h=0,lwd=1,col="gray")
          grid(NA,NULL,lwd=0.8)
          abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
          points(date.time,data,type="p",xlim=date.time[input$slider1],ylim=input$slider2,
                 col=input$integer,lwd=linesize)
          axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
          rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
          axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
          box(col="gray20",lwd=1)
          if (as.logical(input$trend)){
            # calculate the linear trend
            x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
            model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
            lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
            conf <- confint(model,"x",level=0.95)
            trend <- model$coeff["x"]*12.
            lconf <- conf[1]*12.
            uconf <- conf[2]*12
            
            mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
            trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                               format(uconf,digits=2,nsmall=2),"] ",sep="")
            xtext <- as.Date(date.time[input$slider1[1]])
            
            pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            drange <- pmax-pmin
            text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
            text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
          }
          mtext(input$text2)
          on.exit(dev.off())
        }
        
        if (input$checkGroup==3){
          iwidth  <- imagewidth
          iheight <- imageheight
          png(file, width=iwidth, height=iheight)
          par(cex = textsize)
          plot(date.time,data,type="o",xlim=date.time[input$slider1],ylim=input$slider2,
               col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
          abline(h=0,lwd=1,col="gray")
          grid(NA,NULL,lwd=0.8)
          abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
          points(date.time,data,type="o",xlim=date.time[input$slider1],ylim=input$slider2,
                 col=input$integer,lwd=linesize)
          axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
          rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
          axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
          box(col="gray20",lwd=1)
          if (as.logical(input$trend)){
            # calculate the linear trend
            x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
            model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
            lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
            conf <- confint(model,"x",level=0.95)
            trend <- model$coeff["x"]*12.
            lconf <- conf[1]*12.
            uconf <- conf[2]*12
            
            mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
            trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                               format(uconf,digits=2,nsmall=2),"] ",sep="")
            xtext <- as.Date(date.time[input$slider1[1]])
            
            pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            drange <- pmax-pmin
            text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
            text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
          }
          mtext(input$text2)
          on.exit(dev.off())
        }
        
        if (input$checkGroup==4){
          iwidth  <- imagewidth
          iheight <- imageheight
          png(file, width=iwidth, height=iheight)
          par(cex = textsize)
          plot(date.time,data,type="s",xlim=date.time[input$slider1],ylim=input$slider2,
               col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
          abline(h=0,lwd=1,col="gray")
          grid(NA,NULL,lwd=0.8)
          abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
          points(date.time,data,type="s",xlim=date.time[input$slider1],ylim=input$slider2,
                 col=input$integer,lwd=linesize)
          axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
          rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
          axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
          box(col="gray20",lwd=1)
          if (as.logical(input$trend)){
            # calculate the linear trend
            x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
            model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
            lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
            conf <- confint(model,"x",level=0.95)
            trend <- model$coeff["x"]*12.
            lconf <- conf[1]*12.
            uconf <- conf[2]*12
            
            mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
            trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                               format(uconf,digits=2,nsmall=2),"] ",sep="")
            xtext <- as.Date(date.time[input$slider1[1]])
            
            pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            drange <- pmax-pmin
            text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
            text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
          }
          mtext(input$text2)
          on.exit(dev.off())
        }
        
        if (input$checkGroup==5){
          iwidth  <- imagewidth
          iheight <- imageheight
          png(file, width=iwidth, height=iheight)
          par(cex = textsize)
          plot(date.time,data,type="h",xlim=date.time[input$slider1],ylim=input$slider2,
               col="white",main=input$text1,xlab="time",ylab=ylabel,axes=F)
          abline(h=0,lwd=1,col="gray")
          grid(NA,NULL,lwd=0.8)
          abline(v=date.time,col="lightgray",lty=3,lwd=0.8)
          points(date.time,data,type="h",xlim=date.time[input$slider1],ylim=input$slider2,
                 col=input$integer,lwd=linesize)
          axis(side=1,at=date.time[dum_tick],labels=date.lab,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=1,at=date.time[dum_tick],labels=F,tck=0.015,col.ticks="gray20", col.axis="gray20")
          rug(x = date.time[dum_tick2],ticksize=0.015,side=1,quiet=T)
          axis(side=2,tck=-0.025,col.ticks="gray20", col.axis="gray20")
          axis(side=2,tck=0.015,col.ticks="gray20", col.axis="gray20",labels=F)
          box(col="gray20",lwd=1)
          if (as.logical(input$trend)){
            # calculate the linear trend
            x <- c(1:length(date.time[input$slider1[1]:input$slider1[2]]))
            model <- lm(data[input$slider1[1]:input$slider1[2]]~x,na.action=na.exclude)
            lines(date.time[input$slider1[1]:input$slider1[2]],predict(model),col="gray20",lwd=linesize)
            conf <- confint(model,"x",level=0.95)
            trend <- model$coeff["x"]*12.
            lconf <- conf[1]*12.
            uconf <- conf[2]*12
            
            mean.out <- format(mean(data[input$slider1[1]:input$slider1[2]],na.rm=TRUE),digits=3,nsmall=1)
            trend.out <- paste("[",format(lconf,digits=2,nsmall=2),",",format(trend,digits=2,nsmall=2),",",
                               format(uconf,digits=2,nsmall=2),"] ",sep="")
            xtext <- as.Date(date.time[input$slider1[1]])
            
            pmin <- min(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            pmax <- max(data[input$slider1[1]:input$slider1[2]],na.rm=T)
            drange <- pmax-pmin
            text(xtext,pmax-0.01*drange,paste("mean:",mean.out,unit,sep=" "),pos=4,cex=textsize)
            text(xtext,pmax-0.08*drange,paste("linear trend:",trend.out,unit,"/yr",sep=""),pos=4,cex=textsize)
          }
          mtext(input$text2)
          on.exit(dev.off())
        }
      } # end if analyze
      
    })    
})

onStop(function() {
  cat("Thanks for using the CM SAF R Toolbox.\n")
  graphics.off()
})
